@@ -7,15 +7,15 @@ use Carp;
 use Moo;
 
 with 'Ovirt';
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 NAME
 
-Ovirt::VM - Bindings for Ovirt VM API 
+Ovirt::VM - Bindings for oVirt VM API 
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
@@ -36,15 +36,18 @@ Version 0.02
  # return xml output
  print $vm->list_xml; 
  
- # list vm attributes
+ # list vm separated by new line
+ # return attributes specified on 'vm_output_attrs'
  print $vm->list;
  
  # list specific vm 
  print $vm->list('b4738b0f-b73d-4a66-baa8-2ba465d63132');
  
- # create, remove vm
+ # create, remove, get all running vm, stop all vm
  $vm->create('vm1','Default','CentOS7');
  $vm->remove('2d83bb51-9a77-432d-939c-35be207017b9');
+ $vm->get_running();
+ $vm->stop_all();
  
  # start, stop, reboot, migrate vm
  $vm->start     ('b4738b0f-b73d-4a66-baa8-2ba465d63132');
@@ -52,6 +55,24 @@ Version 0.02
  $vm->reboot    ('b4738b0f-b73d-4a66-baa8-2ba465d63132');
  $vm->migrate   ('b4738b0f-b73d-4a66-baa8-2ba465d63132');
 
+ # get vm's statistics
+ $vm->get_statistics($vmid);
+ 
+ # add/remove/list nic and disk
+ $vm->add_disk('6efc0cfa-8495-4a96-93e5-ee490328cf48',  # vm id
+                'virtio',                               # driver interface
+                'cow',                                  # format
+                '1073741824',                           # size
+                'mydisk1',                              # disk name
+                '9b952bdc-b7ec-4673-84b0-477b48945a9a'  # storage domain id
+              );
+
+ $vm->add_nic('6efc0cfa-8495-4a96-93e5-ee490328cf48',   # vm id
+              'virtio',                                 # driver interface
+              'nic1',                                   # nic name
+              'rhevm'                                   # network name
+             );
+
  # Output also available in hash
  # for example to print all vm name and state
  my $hash = $vm->hash_output;
@@ -194,25 +215,6 @@ sub vm_action_output {
     $self->log->debug($xml);
     
     return $xml;
-    
-=comment
-    my $status    = $xml->findnodes('/action/status/state');
-
-    if ($status eq 'complete') {
-        my $jobid = $xml->findnodes('/action/job/@id');
-        print "status   : $status\n";
-        print "jobid    : $jobid\n";
-    }
-    elsif ($status =~ /failed/) {
-        # get fault reason and detail
-        my $reason  = $xml->findnodes('/action/fault/reason');
-        my $detail  = $xml->findnodes('/action/fault/detail');
-    
-        print "status   : $status\n";
-        print "reason   : $reason\n";
-        print "detail   : $detail\n";
-    }
-=cut
 }
 
 =head2 remove
@@ -255,6 +257,304 @@ sub remove {
     $self->vm_action_output($output);
 }
 
+=head2 list_disk
+
+ list vm's disk
+ required argument ('vm id')
+
+=cut
+
+sub list_disk {
+    my $self    = shift;
+    my $vmid    = shift || undef;
+    
+    my $disk_url;
+    
+    # don't proceed if vm not exist
+    my $is_valid = $self->is_vmid_valid($vmid);
+    croak "vm id not found" 
+        unless $is_valid;
+    
+    # set the url
+    $disk_url = $self->url . "/$vmid/disks";
+    
+    my $ua      = LWP::UserAgent->new();
+    my $tx      = $ua->get($disk_url);
+    
+    my $parser = XML::LibXML->new();
+    my $output = $parser->parse_string($tx->decoded_content);
+    
+    $self->log->debug($output);
+    
+    return $output; 
+}
+
+=head2 add_disk
+
+ create vm disk
+ required arguments ('vm id', 'driver type', 'format' , 'size in bytes', 'disk name', 'storage domain id' )
+ 
+ supported driver type :
+ - virtio
+ - ide
+ 
+ supported format:
+ - cow
+ - raw
+ 
+ use Ovirt::Storage to get more information for the storage domain id
+ see 'perldoc Ovirt::Storage' for example
+ 
+ example :
+ $vm->add_disk('6efc0cfa-8495-4a96-93e5-ee490328cf48',  # vm id
+                'virtio',                               # driver interface
+                'cow',                                  # format
+                '1073741824',                           # size
+                'mydisk1',                              # disk name
+                '9b952bdc-b7ec-4673-84b0-477b48945a9a'  # storage domain id
+              );
+
+=cut
+
+sub add_disk {
+    my $self = shift;
+    
+    my ($vmid, $driver, $format, $size, $disk_name, $storage_domain_id)  = @_;
+    
+    my $disk_url;
+    
+    # don't proceed if vm not exist
+    my $is_valid = $self->is_vmid_valid($vmid);
+    croak "vm id not found" 
+        unless $is_valid;
+    
+    # set the url to be post
+    $disk_url = $self->url . "/$vmid/disks";
+    
+    croak "driver type not supported, use virtio/e1000/rtl8139." 
+        unless $driver =~ /\b(virtio|ide)\b/;
+        
+    croak "disk format required"
+        unless $format;
+        
+    croak "disk size required"
+        unless $size;    
+        
+    croak "disk name required"
+        unless $disk_name;
+        
+    croak "storage domain id required"
+        unless $storage_domain_id;    
+    
+    # set xml string to be post
+    my $xml = <<EOF;
+<disk>
+    <name>$disk_name</name>
+    <storage_domains>
+        <storage_domain id="$storage_domain_id"/>  
+    </storage_domains>    
+    <size>$size</size>
+    <interface>$driver</interface>
+    <format>$format</format>
+    <bootable>false</bootable>
+</disk>
+EOF
+    
+    $self->log->debug($xml);
+    
+    # set user agent
+    my $ua      = LWP::UserAgent->new();
+    my $action  = $ua->post($disk_url, Content_Type => 'application/xml', Content => $xml);
+    
+    my $parser = XML::LibXML->new();
+    my $output = $parser->parse_string($action->decoded_content);
+    
+    $self->log->debug($output);
+    
+    return $output; 
+}
+
+=head2 remove_disk
+
+ remove vm disk
+ required arguments ('vm id', 'disk id')
+ use method list_disk to get the 'disk id' for particular vm
+ 
+ example :
+ $vm->remove_disk('6efc0cfa-8495-4a96-93e5-ee490328cf48', '99a4f585-9a39-465b-ae24-0068bd4eaad6');
+ 
+=cut
+
+sub remove_disk {
+    my $self = shift;
+    
+    my ($vmid, $disk_id) = @_;
+    
+    my $disk_url;
+    
+    # don't proceed if vm not exist
+    my $is_valid = $self->is_vmid_valid($vmid);
+    croak "vm id not found" 
+        unless $is_valid;
+    
+    # set the url to be post
+    $disk_url = $self->url . "/$vmid/disks/$disk_id";
+    
+    croak "disk id required"
+        unless $disk_id;
+    
+    $self->log->debug("move action url = $disk_url");
+    
+    # set user agent
+    my $ua      = LWP::UserAgent->new();
+    my $action  = $ua->delete($disk_url);
+    
+    my $parser = XML::LibXML->new();
+    my $output = $parser->parse_string($action->decoded_content);
+    
+    $self->vm_action_output($output);
+}
+
+=head2 list_nic
+
+ list vm's nic
+ required argument ('vm id')
+
+=cut
+
+sub list_nic {
+    my $self    = shift;
+    my $vmid    = shift || undef;
+    
+    my $nic_url;
+    
+    # don't proceed if vm not exist
+    my $is_valid = $self->is_vmid_valid($vmid);
+    croak "vm id not found" 
+        unless $is_valid;
+    
+    # set the url
+    $nic_url = $self->url . "/$vmid/nics";
+    
+    my $ua      = LWP::UserAgent->new();
+    my $tx      = $ua->get($nic_url);
+    
+    my $parser = XML::LibXML->new();
+    my $output = $parser->parse_string($tx->decoded_content);
+    
+    $self->log->debug($output);
+    
+    return $output; 
+}
+
+=head2 add_nic
+
+ create vm nic
+ required arguments ('vm id', 'driver type', 'nic name', 'network name' )
+ 
+ supported driver type :
+ - virtio
+ - e1000
+ - rtl8139
+ 
+ use Ovirt::Network to get more information for the network name
+ see 'perldoc Ovirt::Network' for example
+ 
+ example :
+ $vm->add_nic('6efc0cfa-8495-4a96-93e5-ee490328cf48', 'virtio', 'nic1', 'rhevm');
+
+=cut
+
+sub add_nic {
+    my $self = shift;
+    
+    my ($vmid, $driver, $nic_name, $network_name)  = @_;
+    
+    my $nic_url;
+    
+    # don't proceed if vm not exist
+    my $is_valid = $self->is_vmid_valid($vmid);
+    croak "vm id not found" 
+        unless $is_valid;
+    
+    # set the url to be post
+    $nic_url = $self->url . "/$vmid/nics";
+    
+    croak "driver type not supported, use virtio/e1000/rtl8139." 
+        unless $driver =~ /\b(virtio|e1000|rtl8139)\b/;
+        
+    croak "nic name required"
+        unless $nic_name;
+        
+    croak "network name required"
+        unless $network_name;    
+    
+    # set xml string to be post
+    my $xml = <<EOF;
+<nic>
+  <interface>$driver</interface>
+  <name>$nic_name</name>
+  <network>
+    <name>$network_name</name>
+  </network>
+</nic> 
+EOF
+    
+    $self->log->debug($xml);
+    
+    # set user agent
+    my $ua      = LWP::UserAgent->new();
+    my $action  = $ua->post($nic_url, Content_Type => 'application/xml', Content => $xml);
+    
+    my $parser = XML::LibXML->new();
+    my $output = $parser->parse_string($action->decoded_content);
+    
+    $self->log->debug($output);
+    
+    return $output; 
+}
+
+=head2 remove_nic
+
+ remove vm nic
+ required arguments ('vm id', 'nic id')
+ use method list_nic to get the 'nic id' for particular vm
+ 
+ example :
+ $vm->remove_nic('6efc0cfa-8495-4a96-93e5-ee490328cf48', '82416c65-d8b2-4c82-8134-0e73e5ead624');
+ 
+=cut
+
+sub remove_nic {
+    my $self = shift;
+    
+    my ($vmid, $nic_id) = @_;
+    
+    my $nic_url;
+    
+    # don't proceed if vm not exist
+    my $is_valid = $self->is_vmid_valid($vmid);
+    croak "vm id not found" 
+        unless $is_valid;
+    
+    # set the url to be post
+    $nic_url = $self->url . "/$vmid/nics/$nic_id";
+    
+    croak "nic id required"
+        unless $nic_id;
+    
+    $self->log->debug("move action url = $nic_url");
+    
+    # set user agent
+    my $ua      = LWP::UserAgent->new();
+    my $action  = $ua->delete($nic_url);
+    
+    my $parser = XML::LibXML->new();
+    my $output = $parser->parse_string($action->decoded_content);
+    
+    $self->vm_action_output($output);
+}
+
 =head2 create
 
  create vm using template
@@ -344,21 +644,6 @@ EOF
     $self->log->debug($output);
     
     return $output;
-    
-=comment
-    if ($action->is_success) {
-        my $status    = $output->findnodes('/vm/creation_status/state');
-        print "status   : $status\n";
-    }
-    else {
-        $self->log->debug("LWP Error : " . $action->status_line);
-        
-        my $status    = $output->findnodes('/fault/reason');
-        my $detail    = $output->findnodes('/fault/detail');
-        print "status   : $status\n";
-        print "detail   : $detail\n";
-    }
-=cut
 }
 
 =head2 start
@@ -456,6 +741,101 @@ sub stop {
     $self->vm_action_output($output);
 }
 
+=head2 stop_all
+
+ stop all vm if vm state = 'up'
+ do not set 'id' during initialization because it will not return all vm
+ this will loop using stop method
+ $vm->stop_all();
+ 
+=cut
+
+sub stop_all {
+    my $self = shift;
+    
+    # croak if id is defined because it will not return all vm
+    croak "Don't set id during initialization since it will not return all vm"
+        if $self->id;
+        
+    my $running_vms = $self->get_running();
+    
+    if ($running_vms) {
+        my @vms = split '\n' => $running_vms;
+        
+        for my $vm (@vms) {
+            $self->log->debug($vm);
+            my @vmid = split ',' => $vm;
+            $self->stop($vmid[0]);
+        }
+    }
+    else {
+        print "no vm in up state\n";
+        exit;
+    }
+}
+
+=head2 get_running
+
+ return vmid,name if state = 'up' 
+ each vm separated by new line
+ do not set 'id' during initialization because it will not return all vm
+ $vm->get_running();
+ 
+=cut
+
+sub get_running {
+    my $self    = shift;
+    
+    # store the output and return it at the end
+    my $output;
+    
+    # store id, name to array to be looped
+    my @attrs   = qw |id name|;
+    
+    # store the last element to escape the vm_output_delimeter
+    my $last_element = pop (@attrs);
+    $self->log->debug("last element = $last_element");
+    
+    for my $element_id ( 0 .. $#{ $self->hash_output->{vm} } ) {
+        # check the state first, if up, proceed
+        $self->log->debug("requesting vm state for element $element_id");
+        
+        if (my $state = $self->get_vm_by_element_id($element_id, 'state') || $self->not_available) {
+            
+            if ($state ne 'up') {
+                $self->log->debug("not interested, vm state is = " . $state);
+                
+                # not interested if vm is not up, go to next loop
+                next;
+            }            
+        }
+        
+        # in case there's no any element left, the last element become the only attribute requested
+        if (@attrs) {
+            for my $attr (@attrs) {
+                
+                $self->log->debug("requesting attribute $attr for element $element_id");
+
+                my $attr_output = $self->get_vm_by_element_id($element_id, $attr) || $self->not_available;
+                $output         .= $attr_output . ',';
+                $self->log->debug("output for attribute $attr element $element_id = " . $attr_output);
+            }
+        }
+        
+        #handle last element
+        $self->log->debug("requesting attribute $last_element for element $element_id");
+        
+        if (my $last_output = $self->get_vm_by_element_id($element_id, $last_element) || $self->not_available) {
+            $output .= $last_output . "\n";
+            $self->log->debug("output for attribute $last_element element $element_id = " . $last_output);
+        }
+           
+        #$output .= "\n";
+    }
+    
+    return $output;
+}
+
 =head2 reboot
 
  reboot vm
@@ -626,6 +1006,44 @@ sub is_vmid_valid {
     return 0;
 }
 
+=head2 get_statistics
+
+ required argument ($vmid) or set 'id' during initialization
+ return vm statistics
+ $vm->get_statistics($vmid);
+
+=cut
+
+sub get_statistics {
+    my $self    = shift;
+    my $vmid    = $self->id || shift || die "vm id required\n";
+    
+    my $url     = $self->base_url . $self->vm_url . '/' . $vmid . '/statistics';
+    my $ua      = LWP::UserAgent->new();
+    
+    $self->log->debug($url);
+    my $tx      = $ua->get($url);
+    
+    my $output;
+    
+    if ($tx->is_success) {
+        
+        local $XML::LibXML::skipXMLDeclaration = 1;
+        my $parser  = XML::LibXML->new();
+        $output     = $parser->parse_string($tx->decoded_content);
+    }
+    else {
+        my $err = $tx->status_line;
+        $self->log->debug("LWP Error : " . $err);
+        $self->log->debug("LWP Decoded Content :" . $tx->decoded_content);
+        
+        croak "LWP Status line : " . $err;
+        croak "LWP Decoded Content :" . $tx->decoded_content;
+    }
+    
+    return $output;
+}
+
 =head2 list
  
  return vm's attributes text output from hash_output attribute
